@@ -134,41 +134,56 @@ class ModelBase(object):
         self._targets = OrderedDict()
 
         if isinstance(value, Layer):
-            if not value.is_root or value.nodes.key_list[0] != value.uuid:
-                value.nodes = OrderedDict([(mod.uuid, mod) for mod in list(value.modules()) if isinstance(mod, Layer)])
-                for name, mod in value.named_modules():
-                    if isinstance(mod, Layer):
-                        mod.nodes = value.nodes
-                        mod.relative_name = name
+            # if not value.is_root or value.nodes.key_list[0] != value.uuid:
+            #     value.nodes = OrderedDict([(mod.uuid, mod) for mod in list(value.modules()) if isinstance(mod, Layer)])
+            #     for name, mod in value.named_modules():
+            #         if isinstance(mod, Layer):
+            #             mod.nodes = value.nodes
+            #             mod.relative_name = name
 
-            inp_shape = None
-            if hasattr(value, 'signature') and value.signature is not None and len(value.signature.inputs) > 0:
-                inp_shape = value.input_shape
-            else:
+            inp_shape =self._model.input_shape
+            if inp_shape is None and hasattr(value, 'signature') and value.signature is not None and len(value.signature.inputs) > 0:
+                inp_shape = value._signature.inputs.value_list[0].shape
+            elif inp_shape is None :
                 inp_shape = copy.deepcopy(value.input_shape)
-            self.inputs = OrderedDict()
             self._initial_graph(input_shape=inp_shape, output=value)
         elif isinstance(value, np.ndarray) or 'tensor' in value.__name__.lower():
-            self.inputs = OrderedDict()
             self._initial_graph(input_shape=int_shape(value), output=to_tensor(value))
         else:
             raise ValueError('Only Layer, Module, Image and Tensor can be valid model')
 
     @property
     def inputs(self):
-        return NotImplemented
+        if self._model is not None and callable(self._model):
+            if hasattr(self._model, '_signature'):
+                self._model._signature = get_signature(self._model)
+            return self._model._signature.inputs
+        else:
+            return None
 
     @inputs.setter
     def inputs(self, value):
-        pass
+        if self._model is not None and callable(self._model):
+            if hasattr(self._model, '_signature'):
+                self._model._signature = get_signature(self._model)
+            self._model._signature.inputs = value
 
     @property
     def outputs(self):
-        return NotImplemented
+        if self._model is not None and callable(self._model):
+            if hasattr(self._model, '_signature'):
+                self._model._signature = get_signature(self._model)
+            return self._model._signature.outputs
+        else:
+            return None
 
     @outputs.setter
     def outputs(self, value):
-        pass
+        if self._model is not None and callable(self._model):
+            if hasattr(self._model, '_signature'):
+                self._model._signature = get_signature(self._model)
+            self._model._signature.outputs = value
+
 
     @property
     def targets(self):
