@@ -36,6 +36,7 @@ _valid_when=["on_training_start"
  ,"on_metrics_evaluation_end"
  ,"on_progress_start"
  ,"on_progress_end"
+,"on_excution_exception"
  ,"on_model_saving_start"
  ,"on_model_saving_end"]
 
@@ -251,7 +252,7 @@ class CallbackBase(ABC):
 
     def on_progress_start(self, training_context):
         """
-        Called at the beginning of the evaluation step.
+        Called at the beginning of the printing progress step.
         :param training_context: Dict containing information regarding the training process.
 
         """
@@ -259,7 +260,7 @@ class CallbackBase(ABC):
         pass
     def on_progress_end(self, training_context):
         """
-        Called at the end of the evaluation step.
+        Called at the end of the printing progress step.
 
 
 
@@ -269,7 +270,17 @@ class CallbackBase(ABC):
 
         pass
 
+    def on_excution_exception(self, training_context):
+        """
+        Called when the expection occure.
 
+
+
+        :param training_context: Dict containing information regarding the training process.
+
+        """
+
+        pass
     def on_model_saving_start(self, training_context):
         """
         Called just before the optimization step.
