@@ -601,7 +601,7 @@ class PrintGradientsCallback(VisualizationCallbackBase):
                     if last_layer_name != '' and not self.is_modulefict:
                         self.last_layer[last_layer_name] = 'last_layer'
 
-                grads_and_vars = list(training_context['optimizer'].grads_and_vars)
+                grads_and_vars = list(training_context['grads_and_vars'])
                 grads_dict = OrderedDict()
                 for grad, var in grads_and_vars:
                     grads_dict[var.ref()] = to_numpy(grad)
