@@ -402,14 +402,14 @@ def load_folder_images(dataset_name='', base_folder=None, classes=None, shuffle=
             labelsdata = LabelDataset(labels,object_type=ObjectType.classification_label)
             labelsdata.binding_class_names(class_names)
 
-            traindata = Iterator(data=imagedata, label=labelsdata)
+            traindata = Iterator(data=imagedata, label=labelsdata,is_shuffle=shuffle)
             dataset = DataProvider(dataset_name, traindata=traindata)
             dataset.binding_class_names(class_names)
 
         else:
             imgs = list_images(base_folder)
             imagedata = ImageDataset(imgs, object_type=ObjectType.rgb)
-            traindata = Iterator(data=imagedata)
+            traindata = Iterator(data=imagedata,is_shuffle=shuffle)
             dataset = DataProvider(dataset_name, traindata=traindata)
         return dataset
     else:
