@@ -5,7 +5,6 @@ from __future__ import print_function
 
 import math
 import numbers
-import random
 import warnings
 from typing import Optional, Tuple, overload
 
@@ -16,13 +15,13 @@ from torch._jit_internal import List
 from torch.nn import init
 from torch.nn.parameter import Parameter
 from torch.nn.utils.rnn import PackedSequence
+
+from trident.backend.common import *
+from trident.backend.pytorch_backend import Layer, get_device
+from trident.backend.pytorch_ops import *
 from trident.layers.pytorch_layers import Embedding, Dense, SoftMax
 
-from trident.backend.pytorch_ops import *
-from  trident.backend.common import *
-from trident.backend.pytorch_backend import Layer, get_device
-
-__all__ = ['RNNBase','RNN','LSTM','GRU','LSTMDecoder']
+__all__ = ['RNNBase', 'RNN', 'LSTM', 'GRU', 'LSTMDecoder']
 _rnn_impls = {
     'RNN_TANH': _VF.rnn_tanh,
     'RNN_RELU': _VF.rnn_relu,
@@ -33,7 +32,6 @@ def apply_permutation(tensor: Tensor, permutation: Tensor, dim: int = 1) -> Tens
     return tensor.index_select(dim, permutation)
 
 
-
 class RNNBase(Layer):
     __constants__ = ['mode', 'input_filters', 'hidden_size', 'num_layers', 'use_bias',
                      'batch_first', 'dropout_rate', 'bidirectional']
@@ -50,18 +48,19 @@ class RNNBase(Layer):
     filter_index: int
 
     def __init__(self, mode: str, hidden_size: int,
-                 num_layers: int = 1,stateful=False, use_bias: bool = True, batch_first: bool = False,
-                 dropout_rate: float = 0., bidirectional: bool = False,name=None,keep_output=False,in_sequence=True,filter_index=-1) -> None:
-        super(RNNBase, self).__init__(name=name,keep_output=keep_output)
+                 num_layers: int = 1, stateful=False, use_bias: bool = True, batch_first: bool = False,
+                 dropout_rate: float = 0., bidirectional: bool = False, proj_size: int = 0, name=None, keep_output=False, in_sequence=True, filter_index=-1) -> None:
+        super(RNNBase, self).__init__(name=name, keep_output=keep_output)
 
         self.mode = mode
         self.hidden_size = hidden_size
+        self.proj_size = proj_size
         self.num_layers = num_layers
         self.use_bias = use_bias
-        self.stateful=stateful
+        self.stateful = stateful
         self._batch_first = batch_first
         if not self._batch_first:
-            self.batch_index =1
+            self.batch_index = 1
         else:
             self.batch_index = 0
         self.filter_index = -1
@@ -107,10 +106,10 @@ class RNNBase(Layer):
             else:
                 self.batch_index = 0
 
-    def initial_state(self,input) :
+    def initial_state(self, input):
         pass
 
-    def build(self, input_shape:TensorShape):
+    def build(self, input_shape: TensorShape):
         if not self._built:
             for layer in range(self.num_layers):
                 for direction in range(self.num_directions):
@@ -144,7 +143,6 @@ class RNNBase(Layer):
             self.flatten_parameters()
             self.reset_parameters()
 
-
     # def __setattr__(self, attr, value):
     #     if hasattr(self, "_flat_weights_names") and attr in self._flat_weights_names:
     #         # keep self._flat_weights up to date if you do self.weight = ...
@@ -198,8 +196,8 @@ class RNNBase(Layer):
                         num_weights += 1
                     torch._cudnn_rnn_flatten_weight(
                         self._flat_weights, num_weights,
-                        self.input_size, rnn.get_cudnn_mode(self.mode),
-                        self.input_filters, self.proj_size, self.num_layers,  # type: ignore
+                        self.input_filters, rnn.get_cudnn_mode(self.mode),
+                        self.hidden_size, self.proj_size, self.num_layers,  # type: ignore
                         self.batch_first, bool(self.bidirectional))  # type: ignore
 
     def _apply(self, fn):
@@ -237,7 +235,7 @@ class RNNBase(Layer):
         else:
             mini_batch = input.size(0) if self.batch_first else input.size(1)
         num_directions = 2 if self.bidirectional else 1
-        expected_hidden_size = (self.num_layers * num_directions,  mini_batch, self.hidden_size)
+        expected_hidden_size = (self.num_layers * num_directions, mini_batch, self.hidden_size)
         return expected_hidden_size
 
     def check_hidden_size(self, hx: Tensor, expected_hidden_size: Tuple[int, int, int],
@@ -331,7 +329,6 @@ class RNNBase(Layer):
                     self._flat_weights_names.extend(weights[:2])
         self._flat_weights = [(lambda wn: getattr(self, wn) if hasattr(self, wn) else None)(wn) for wn in self._flat_weights_names]
 
-
     @property
     def all_weights(self) -> List[Parameter]:
         return [[getattr(self, weight) for weight in weights] for weights in self._all_weights]
@@ -465,7 +462,8 @@ class RNN(RNNBase):
 #
 # TODO: remove the overriding implementations for LSTM and GRU when TorchScript
 # support expressing these two modules generally.
-from torch.nn.modules.rnn import LSTM
+
+
 
 class LSTM(RNNBase):
     r"""Applies a multi-layer long short-term memory (LSTM) RNN to an input
@@ -572,36 +570,32 @@ class LSTM(RNNBase):
         >>> output, (hn, cn) = rnn(input, (h0, c0))
     """
 
-    def __init__(self, hidden_size,num_layers:int =2,activation=None,stateful=False,use_bias=False,use_attention=False,attention_size=16,batch_first=False,dropout_rate=0,bidirectional=False,name=None,keep_output=False, in_sequence=True,filter_index=-1, **kwargs):
+    def __init__(self, hidden_size, num_layers: int = 2, activation=None, stateful=False, use_bias=False, use_attention=False, attention_size=16, batch_first=False, dropout_rate=0,
+                 bidirectional=False, name=None, keep_output=False, in_sequence=True, filter_index=-1, **kwargs):
         super(LSTM, self).__init__(mode='LSTM', hidden_size=hidden_size,
-                 num_layers=num_layers, stateful=stateful,use_bias=use_bias, batch_first=batch_first,
-                 dropout_rate=dropout_rate, bidirectional=bidirectional,name=name,keep_output=keep_output,in_sequence=in_sequence,filter_index=filter_index)
+                                   num_layers=num_layers, stateful=stateful, use_bias=use_bias, batch_first=batch_first,
+                                   dropout_rate=dropout_rate, bidirectional=bidirectional, name=name, keep_output=keep_output, in_sequence=in_sequence, filter_index=filter_index)
 
         self.mode = 'LSTM'
-        self.use_attention=use_attention
-        self.attention_size=attention_size
-        self.hidden_state=None
-        self.cell_state=None
+        self.use_attention = use_attention
+        self.attention_size = attention_size
+        self.hidden_state = None
+        self.cell_state = None
 
+        self.stateful = stateful
 
-        self.stateful=stateful
-
-
-
-    def initial_state(self,input) :
+    def initial_state(self, input):
         max_batch_size = input.size(0) if self.batch_first else input.size(1)
         num_directions = 2 if self.bidirectional else 1
         zeros = torch.zeros(self.num_layers * num_directions,
-                                max_batch_size, self.hidden_size,
-                                dtype=dtype.float32, requires_grad=False).to(get_device())
-        self.hidden_state=zeros
+                            max_batch_size, self.hidden_size,
+                            dtype=dtype.float32, requires_grad=False).to(get_device())
+        self.hidden_state = zeros
         self.cell_state = zeros
 
-
-
     def clear_state(self):
-        self.hidden_state= zeros_like(self.hidden_state,dtype=dtype.float32, requires_grad=False ).to(get_device())
-        self.cell_state= zeros_like(self.cell_state,dtype=dtype.float32, requires_grad=False).to(get_device())
+        self.hidden_state = zeros_like(self.hidden_state, dtype=dtype.float32, requires_grad=False).to(get_device())
+        self.cell_state = zeros_like(self.cell_state, dtype=dtype.float32, requires_grad=False).to(get_device())
 
     def check_forward_args(self, input: Tensor, hidden: Tuple[Tensor, Tensor], batch_sizes: Optional[Tensor]):
         self.check_input(input, batch_sizes)
@@ -644,10 +638,10 @@ class LSTM(RNNBase):
         pass
 
     def forward(self, *x, **kwargs):  # noqa: F811
-        x=unpack_singleton(x)
-        if isinstance(x,tuple):
-            x,hx=x
-        orig_input =x
+        x = unpack_singleton(x)
+        if isinstance(x, tuple):
+            x, hx = x
+        orig_input = x
         self.flatten_parameters()
         # xxx: isinstance check needs to be in conditional for TorchScript to compile
         if isinstance(orig_input, PackedSequence):
@@ -656,14 +650,13 @@ class LSTM(RNNBase):
             max_batch_size = int(max_batch_size)
         else:
             if not self.batch_first:
-                x = x.transpose(1,0)
+                x = x.transpose(1, 0)
             batch_sizes = None
             max_batch_size = x.size(0) if self.batch_first else x.size(1)
             sorted_indices = None
             unsorted_indices = None
 
-
-        if self.hidden_state is None or self.cell_state is None or max_batch_size!=int_shape(self.hidden_state)[1]:
+        if self.hidden_state is None or self.cell_state is None or max_batch_size != int_shape(self.hidden_state)[1]:
             self.initial_state(x)
         else:
             if not self.stateful:
@@ -679,11 +672,10 @@ class LSTM(RNNBase):
             result = _VF.lstm(x, batch_sizes, (self.hidden_state, self.cell_state), self._flat_weights, self.use_bias,
                               self.num_layers, self.dropout_rate, self.training, self.bidirectional)
 
-
         output = result[0].permute(1, 0, 2) if self.batch_first == False else result[0]
-        #hidden = result[1:]
-        self.hidden_state=result[1:][0].detach()
-        self.cell_state=result[1:][1].detach()
+        # hidden = result[1:]
+        self.hidden_state = result[1:][0].detach()
+        self.cell_state = result[1:][1].detach()
         if self.use_attention:
             output = self.attention(output)
 
@@ -695,20 +687,22 @@ class LSTM(RNNBase):
 
             return output, self.permute_hidden((self.hidden_state, self.cell_state), unsorted_indices)
 
+
 class LSTMDecoder(Layer):
-    def __init__(self, num_chars, embedding_dim, h_size=512, num_layers=2,sequence_length=128,stateful=True, dropout_rate=0.2,bidirectional=False,use_attention=False,attention_size=16,teacher_forcing_ratio=1):
+    def __init__(self, num_chars, embedding_dim, h_size=512, num_layers=2, sequence_length=128, stateful=True, dropout_rate=0.2, bidirectional=False, use_attention=False,
+                 attention_size=16, teacher_forcing_ratio=1):
         super().__init__()
-        self.teacher_forcing_ratio=teacher_forcing_ratio
+        self.teacher_forcing_ratio = teacher_forcing_ratio
         self.num_chars = num_chars
-        self.embedding_dim=embedding_dim
+        self.embedding_dim = embedding_dim
         self.h_size = h_size
         self.num_layers = num_layers
-        self.sequence_length=sequence_length
+        self.sequence_length = sequence_length
         self.embedding = Embedding(embedding_dim=256, num_embeddings=num_chars, sparse=False, norm_type=2, add_noise=True, noise_intensity=0.12)
-        self.lstm = LSTM(hidden_size=h_size, num_layers=num_layers, stateful=stateful, batch_first=False, dropout_rate=dropout_rate, bidirectional=bidirectional, use_attention=use_attention, attention_size=attention_size)
-        self.fc_out =Dense(num_chars,use_bias=False,activation=leaky_relu)
-        self.softmax=SoftMax(axis=-1)
-
+        self.lstm = LSTM(hidden_size=h_size, num_layers=num_layers, stateful=stateful, batch_first=False, dropout_rate=dropout_rate, bidirectional=bidirectional,
+                         use_attention=use_attention, attention_size=attention_size)
+        self.fc_out = Dense(num_chars, use_bias=False, activation=leaky_relu)
+        self.softmax = SoftMax(axis=-1)
 
     def forward(self, input: Tensor, hx: Optional[Tuple[Tensor, Tensor]] = None
                 ) -> Tuple[Tensor, Tuple[Tensor, Tensor]]:  # noqa: F811
@@ -722,23 +716,20 @@ class LSTMDecoder(Layer):
         # n directions in the decoder will both always be 1, therefore:
         # hidden = [n layers, batch size, hid dim]
         # context = [n layers, batch size, hid dim]
-        x,(self.hidden_state, self.cell_state) =unpack_singleton(x)
-        B,N,C=int_shape(x)
-        outputs =[]
+        x, (self.hidden_state, self.cell_state) = unpack_singleton(x)
+        B, N, C = int_shape(x)
+        outputs = []
         # input = [batch size,1]
 
-
-        decoder_input =expand_dims(x[:,-1,:] ,1) # shape: (batch_size, input_size)
+        decoder_input = expand_dims(x[:, -1, :], 1)  # shape: (batch_size, input_size)
         decoder_hidden = (self.hidden_state, self.cell_state)
 
         # predict recursively
         for t in range(self.sequence_length):
-            decoder_output, decoder_hidden =  self.lstm(decoder_input, decoder_hidden)
-            outputs.append(self.softmax(self.fc_out (decoder_output.squeeze(1))))
+            decoder_output, decoder_hidden = self.lstm(decoder_input, decoder_hidden)
+            outputs.append(self.softmax(self.fc_out(decoder_output.squeeze(1))))
             decoder_input = decoder_output
-        return stack(outputs,1)
-
-
+        return stack(outputs, 1)
 
 
 class GRU(RNNBase):
@@ -846,19 +837,19 @@ class GRU(RNNBase):
         >>> output, hn = rnn(input, h0)
     """
 
-    def __init__(self, hidden_size,num_layers:int =2,activation=None,stateful=False,use_bias=False,use_attention=False,attention_size=16,batch_first=False,dropout_rate=0,bidirectional=False,name=None,keep_output=False, in_sequence=True,filter_index=-1, **kwargs):
+    def __init__(self, hidden_size, num_layers: int = 2, activation=None, stateful=False, use_bias=False, use_attention=False, attention_size=16, batch_first=False, dropout_rate=0,
+                 bidirectional=False, name=None, keep_output=False, in_sequence=True, filter_index=-1, **kwargs):
         super(GRU, self).__init__(mode='GRU', hidden_size=hidden_size,
-                 num_layers=num_layers, stateful=stateful,use_bias=use_bias, batch_first=batch_first,
-                 dropout_rate=dropout_rate, bidirectional=bidirectional,name=name,keep_output=keep_output,in_sequence=in_sequence,filter_index=filter_index)
+                                  num_layers=num_layers, stateful=stateful, use_bias=use_bias, batch_first=batch_first,
+                                  dropout_rate=dropout_rate, bidirectional=bidirectional, name=name, keep_output=keep_output, in_sequence=in_sequence, filter_index=filter_index)
 
         self.mode = 'GRU'
-        self.use_attention=use_attention
-        self.attention_size=attention_size
-        self.hidden_state=None
-        self.stateful=stateful
+        self.use_attention = use_attention
+        self.attention_size = attention_size
+        self.hidden_state = None
+        self.stateful = stateful
 
-
-    def initial_state(self,input) :
+    def initial_state(self, input):
         max_batch_size = input.size(0) if self.batch_first else input.size(1)
         num_directions = 2 if self.bidirectional else 1
 
@@ -866,8 +857,6 @@ class GRU(RNNBase):
                                         max_batch_size, self.hidden_size,
                                         dtype=self.weights[0].dtype, device=self.weights[0].device, requires_grad=False)
 
-
-
     @overload
     @torch._jit_internal._overload_method  # noqa: F811
     def forward(self, input: Tensor, hx: Optional[Tensor] = None) -> Tuple[Tensor, Tensor]:  # noqa: F811
@@ -878,7 +867,7 @@ class GRU(RNNBase):
     def forward(self, input: PackedSequence, hx: Optional[Tensor] = None) -> Tuple[PackedSequence, Tensor]:  # noqa: F811
         pass
 
-    def forward(self,x):  # noqa: F811
+    def forward(self, x):  # noqa: F811
         x = unpack_singleton(x)
         if isinstance(x, tuple):
             x, hx = x
@@ -898,12 +887,11 @@ class GRU(RNNBase):
             sorted_indices = None
             unsorted_indices = None
 
-        if self.stateful==False or self.hidden_state is None  or max_batch_size!=int_shape(self.hidden_state)[1]:
+        if self.stateful == False or self.hidden_state is None or max_batch_size != int_shape(self.hidden_state)[1]:
             self.initial_state(x)
 
         else:
-            self.hidden_state= self.permute_hidden(self.hidden_state, sorted_indices)
-
+            self.hidden_state = self.permute_hidden(self.hidden_state, sorted_indices)
 
         self.check_forward_args(x, self.hidden_state, batch_sizes)
         if batch_sizes is None:
