@@ -16,7 +16,7 @@ import numpy as np
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-from torch._six import container_abcs
+from collections import abc
 from torch.nn import init
 from torch.nn.parameter import Parameter
 from trident.data.vision_transforms import Resize,Normalize
@@ -201,8 +201,9 @@ def EfficientNet(width_coefficient, depth_coefficient, input_shape, dropout_rate
                             Conv2d_Block((1, 1), round_filters(1280), strides=1, use_bias=False, auto_pad=True,
                                          padding_mode='zero', normalization='batch', activation='swish',
                                          name='top_conv'))
-    efficientnet.add_module('avg_pool', GlobalAvgPool2d(name='avg_pool'))
+
     if include_top:
+        efficientnet.add_module('avg_pool', GlobalAvgPool2d(name='avg_pool'))
         if dropout_rate > 0:
             efficientnet.add_module('top_dropout', Dropout(dropout_rate, name='top_dropout'))
         efficientnet.add_module('fc', Dense(num_classes, activation=None, name='fc'))
