@@ -16,7 +16,7 @@ import numpy as np
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
-from torch._six import container_abcs
+from collections import abc
 from torch.nn import init
 from torch.nn.parameter import Parameter
 from trident.models.pretrained_utils import _make_recovery_model_include_top
@@ -143,8 +143,9 @@ def SE_ResNet(block, layers, input_shape=(3, 224, 224), num_classes=1000, use_bi
     resnet.add_module('layer2',(_make_layer(block, 128, layers[1], strides=2, dilate=None,use_bias=use_bias,layer_name='layer2' )))
     resnet.add_module('layer3',(_make_layer(block, 256, layers[2], strides=2, dilate=None,use_bias=use_bias,layer_name='layer3' )))
     resnet.add_module('layer4' ,(_make_layer(block, 512, layers[3], strides=2, dilate=None,use_bias=use_bias,layer_name='layer4' )))
-    resnet.add_module('avg_pool',GlobalAvgPool2d(name='avg_pool'))
+
     if include_top:
+        resnet.add_module('avg_pool', GlobalAvgPool2d(name='avg_pool'))
         resnet.add_module('fc',Dense(num_classes,activation=None,name='fc'))
         resnet.add_module('softmax', SoftMax(name='softmax'))
     resnet.name=model_name
@@ -176,10 +177,10 @@ def SE_ResNet50(include_top=True,
         recovery_model=load(os.path.join(dirname,'se_resnet50.pth'))
         recovery_model = fix_layer(recovery_model)
         recovery_model.name = 'se_resnet50'
-        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes,default_shape=(3,224,224),input_shape=input_shape, freeze_features=freeze_features)
         resnet50.model = recovery_model
     else:
-        resnet50.model = _make_recovery_model_include_top(resnet50.model, include_top=include_top, classes=classes, freeze_features=True)
+        resnet50.model = _make_recovery_model_include_top(resnet50.model, include_top=include_top, classes=classes,default_shape=(3,224,224),input_shape=input_shape, freeze_features=True)
 
     resnet50.model.input_shape = input_shape
     resnet50.model.to(_device)
