@@ -1,7 +1,6 @@
 import torch
-from collections import defaultdict
-import collections.abc
-container_abcs = collections.abc
+from collections import defaultdict,abc
+
 import warnings
 from enum import Enum
 from typing import Any, Dict, List, Optional, Tuple
@@ -197,7 +196,7 @@ class GradScaler(object):
                     assert self._scale is not None
                     stash.append(_MultiDeviceReplicator(self._scale))
                 return val * stash[0].get(val.device)
-            elif isinstance(val, container_abcs.Iterable):
+            elif isinstance(val, abc.Iterable):
                 iterable = map(apply_scale, val)
                 if isinstance(val, list) or isinstance(val, tuple):
                     return type(val)(iterable)
