@@ -242,16 +242,17 @@ class _ClassificationLoss(Loss):
         if target.dtype == str2dtype('long'):
             self.is_target_onehot = False
         elif target.dtype != str2dtype('long') and (target.min() >= 0 and target.max() <= 1 and abs(output_exp.sum(-1).mean() - 1) < 1e-4):
-            target = clip(target, min=1e-8, max=1 - 1e-8)
+            target = clip(target, min=1e-7, max=1 - 1e-7)
             self.is_target_onehot = True
 
         # need target onehot but currently not
         if target.dtype == torch.long and self.need_target_onehot == True and self.is_target_onehot == False:
             target = make_onehot(target, num_classes=self.num_classes, axis=self.axis).to(get_device())
+            target = clip(target, min=1e-7, max=1 - 1e-7)
             target.require_grads = False
             self.is_target_onehot = True
             if self.label_smooth:
-                target = target * (torch.Tensor(target.size()).uniform_(0.9, 1).to(output.device))
+                target = clip(target + random_uniform_like(target,-0.2,0.2).to(target.device),1e-7,1.1)
                 self.is_target_onehot = True
             target.require_grads = False
 
@@ -606,9 +607,9 @@ class CrossEntropyLoss(_ClassificationLoss):
                 sample_weight = self.sample_weight.view(*reshape_shape) * self.ignore_index_weight.view(*reshape_shape)
 
             if not self.is_logsoftmax:
-                return -reduce_sum(target * F.log_softmax(output, dim=self.axis) * sample_weight, axis=1)
+                return -reduce_sum(target * F.log_softmax(output, dim=self.axis) * sample_weight, axis=self.axis)
             else:
-                return -reduce_sum(target * output * sample_weight, axis=1)
+                return -reduce_sum(target * output * sample_weight, axis=self.axis)
         return loss
 
 
