@@ -94,7 +94,7 @@ def accuracy(output:Tensor, target:Tensor, topk:int=1,axis:int=1,ignore_index:Un
 
 
 @torch.no_grad()
-def recall(output:Tensor, target:Tensor, axis=1,ignore_index=-100):
+def recall(output:Tensor, target:Tensor, axis=1,ignore_index=0):
     """Computes the precision@k for the specified values of k
     prec1, prec5 = accuracy(output.data, target, topk=(1, 5))
     """
