@@ -1104,17 +1104,23 @@ class Ranger(Optimizer):
         """
         loss = None
 
+
         if closure is not None:
             with torch.enable_grad():
                 loss = closure()
 
-        # Evaluate averages and grad, update param tensors
         for group in self.param_groups:
             for p in group['params']:
                 if p.grad is None or not p.requires_grad:
                     continue
+                # cast data type
+                half_precision = False
+                if p.data.dtype == torch.float16:
+                    half_precision = True
+                    p.data = p.data.float()
+                    p.grad = p.grad.float()
 
-                grad = p.grad.data.float()
+                grad = p.grad.data
 
                 if grad.is_sparse:
                     raise RuntimeError('Ranger optimizer does not support sparse gradients')
@@ -1264,109 +1270,115 @@ class RangerLars(Optimizer):
             with torch.enable_grad():
                 loss = closure()
 
-            for group in self.param_groups:
-                for p in group['params']:
-                    if p.grad is None or not p.requires_grad:
-                        continue
-                    grad = p.grad.data.float()
+        for group in self.param_groups:
+            for p in group['params']:
+                if p.grad is None or not p.requires_grad:
+                    continue
+                # cast data type
+                half_precision = False
+                if p.data.dtype == torch.float16:
+                    half_precision = True
+                    p.data = p.data.float()
+                    p.grad = p.grad.float()
+                grad = p.grad.data.float()
 
-                    if grad.is_sparse:
-                        raise RuntimeError('Ranger optimizer does not support sparse gradients')
+                if grad.is_sparse:
+                    raise RuntimeError('Ranger optimizer does not support sparse gradients')
 
-                    p_data = p.data.float()
+                p_data = p.data.float()
 
-                    state = self.state[p]  # get state dict for this param
+                state = self.state[p]  # get state dict for this param
 
-                    if len(state) == 0:  # if first time to run...init dictionary with our desired entries
-                        state['step'] = 0
-                        state['exp_avg'] = torch.zeros_like(p_data, memory_format=torch.preserve_format)
-                        state['exp_avg_sq'] = torch.zeros_like(p_data, memory_format=torch.preserve_format)
+                if len(state) == 0:  # if first time to run...init dictionary with our desired entries
+                    state['step'] = 0
+                    state['exp_avg'] = torch.zeros_like(p_data, memory_format=torch.preserve_format)
+                    state['exp_avg_sq'] = torch.zeros_like(p_data, memory_format=torch.preserve_format)
 
-                        # look ahead weight storage now in state dict
-                        state['slow_buffer'] = torch.empty_like(p.data)
-                        state['slow_buffer'].copy_(p.data)
+                    # look ahead weight storage now in state dict
+                    state['slow_buffer'] = torch.empty_like(p.data)
+                    state['slow_buffer'].copy_(p.data)
+                else:
+                    state['exp_avg'] = state['exp_avg'].type_as(p_data)
+                    state['exp_avg_sq'] = state['exp_avg_sq'].type_as(p_data)
+
+                # begin computations
+                exp_avg, exp_avg_sq = state['exp_avg'], state['exp_avg_sq']
+                beta1, beta2 = group['betas']
+                if self.gradient_centralization in ['all', 'gcc']:
+                    if len(list(grad.size())) > 3:
+                        grad.add_(-grad.mean(dim=tuple(range(1, grad.dim())), keepdim=True))
+
+                exp_avg.mul_(beta1).add_(grad, alpha=1 - beta1)
+
+                exp_avg_sq.mul_(beta2).addcmul_(grad, grad, value=1 - beta2)
+
+                # grad=_filter_grads(grad,self.gradient_centralization)
+
+                state['step'] += 1
+                buffered = self.radam_buffer[int(state['step'] % 10)]
+                if state['step'] == buffered[0]:
+                    N_sma, step_size = buffered[1], buffered[2]
+                else:
+                    buffered[0] = state['step']
+                    beta2_t = beta2 ** state['step']
+                    N_sma_max = 2 / (1 - beta2) - 1
+                    N_sma = N_sma_max - 2 * state['step'] * beta2_t / (1 - beta2_t)
+                    buffered[1] = N_sma
+
+                    # more conservative since it's an approximated value
+                    if N_sma >= self.N_sma_threshhold:
+                        step_size = math.sqrt((1 - beta2_t) * (N_sma - 4) / (N_sma_max - 4) * (N_sma - 2) / N_sma * N_sma_max / (
+                                N_sma_max - 2)) / (1 - beta1 ** state['step'])
                     else:
-                        state['exp_avg'] = state['exp_avg'].type_as(p_data)
-                        state['exp_avg_sq'] = state['exp_avg_sq'].type_as(p_data)
+                        step_size = 1.0 / (1 - beta1 ** state['step'])
+                    buffered[2] = step_size
 
-                    # begin computations
-                    exp_avg, exp_avg_sq = state['exp_avg'], state['exp_avg_sq']
-                    beta1, beta2 = group['betas']
-                    if self.gradient_centralization in ['all', 'gcc']:
-                        if len(list(grad.size())) > 3:
-                            grad.add_(-grad.mean(dim=tuple(range(1, grad.dim())), keepdim=True))
+                update = zeros_like(p_data)
+                if N_sma >= 5:
+                    denom = exp_avg_sq.sqrt().add_(group['eps'])
+                    update.addcdiv_(exp_avg, denom, value=step_size)
+                else:
+                    update.add_(exp_avg, alpha=step_size)
 
-                    exp_avg.mul_(beta1).add_(grad, alpha=1 - beta1)
+                # if group['weight_decay'] != 0:
+                #     update.add_(group['weight_decay'], p_data)
+                if group['weight_decay'] != 0:
+                    grad = grad.add(p, alpha=group['weight_decay'])
 
-                    exp_avg_sq.mul_(beta2).addcmul_(grad, grad, value=1 - beta2)
+                if self.gradient_centralization in ['all', 'gc']:
+                    if len(list(grad.size())) > 1:
+                        grad.add_(-grad.mean(dim=tuple(range(1, len(list(grad.size())))), keepdim=True))
 
-                    # grad=_filter_grads(grad,self.gradient_centralization)
+                radam_norm = update.pow(2.0).sum().sqrt()
+                weight_norm = p.data.pow(2.0).sum().sqrt()
+                if weight_norm == 0 or radam_norm == 0:
+                    trust_ratio = 1.0
+                else:
+                    trust_ratio = weight_norm / radam_norm
 
-                    state['step'] += 1
-                    buffered = self.radam_buffer[int(state['step'] % 10)]
-                    if state['step'] == buffered[0]:
-                        N_sma, step_size = buffered[1], buffered[2]
-                    else:
-                        buffered[0] = state['step']
-                        beta2_t = beta2 ** state['step']
-                        N_sma_max = 2 / (1 - beta2) - 1
-                        N_sma = N_sma_max - 2 * state['step'] * beta2_t / (1 - beta2_t)
-                        buffered[1] = N_sma
+                trust_ratio = clip(to_tensor(trust_ratio), 0.0, 10.0)
 
-                        # more conservative since it's an approximated value
-                        if N_sma >= self.N_sma_threshhold:
-                            step_size = math.sqrt((1 - beta2_t) * (N_sma - 4) / (N_sma_max - 4) * (N_sma - 2) / N_sma * N_sma_max / (
-                                    N_sma_max - 2)) / (1 - beta1 ** state['step'])
-                        else:
-                            step_size = 1.0 / (1 - beta1 ** state['step'])
-                        buffered[2] = step_size
+                state['weight_norm'] = weight_norm
+                state['adam_norm'] = radam_norm
+                state['trust_ratio'] = trust_ratio
 
-                    update = zeros_like(p_data)
-                    if N_sma >= 5:
-                        denom = exp_avg_sq.sqrt().add_(group['eps'])
-                        update.addcdiv_(exp_avg, denom, value=step_size)
-                    else:
-                        update.add_(exp_avg, alpha=step_size)
+                p_data.add_(-update * trust_ratio * group['lr'])
 
-                    # if group['weight_decay'] != 0:
-                    #     update.add_(group['weight_decay'], p_data)
-                    if group['weight_decay'] != 0:
-                        grad = grad.add(p, alpha=group['weight_decay'])
+                if any_abnormal_number(p_data):
+                    sys.stderr.write('{0} p_data has abnormal value,trident automatically replace these abnormal value to zero.\n\r'.format(self.__class__.__name__))
+                p_data.copy_(where(is_abnormal_number(p_data), p.data, p_data))
 
-                    if self.gradient_centralization in ['all', 'gc']:
-                        if len(list(grad.size())) > 1:
-                            grad.add_(-grad.mean(dim=tuple(range(1, len(list(grad.size())))), keepdim=True))
+                p.data.copy_(p_data)
 
-                    radam_norm = update.pow(2.0).sum().sqrt()
-                    weight_norm = p.data.pow(2.0).sum().sqrt()
-                    if weight_norm == 0 or radam_norm == 0:
-                        trust_ratio = 1.0
-                    else:
-                        trust_ratio = weight_norm / radam_norm
-
-                    trust_ratio = clip(to_tensor(trust_ratio), 0.0, 10.0)
-
-                    state['weight_norm'] = weight_norm
-                    state['adam_norm'] = radam_norm
-                    state['trust_ratio'] = trust_ratio
-
-                    p_data.add_(-update * trust_ratio * group['lr'])
-
-                    if any_abnormal_number(p_data):
-                        sys.stderr.write('{0} p_data has abnormal value,trident automatically replace these abnormal value to zero.\n\r'.format(self.__class__.__name__))
-                    p_data.copy_(where(is_abnormal_number(p_data), p.data, p_data))
-
-                    p.data.copy_(p_data)
-
-                    # integrated look ahead...
-                    # we do it at the param level instead of group level
-                    if state['step'] % group['k'] == 0:
-                        slow_p = state['slow_buffer']  # get access to slow param tensor
-                        slow_p.add_(p.data - slow_p, alpha=self.alpha)  # (fast weights - slow weights) * alpha
-                        if any_abnormal_number(slow_p):
-                            sys.stderr.write('{0} p_data has abnormal value,trident automatically replace these abnormal value to zero.\n'.format(self.__class__.__name__))
-                        slow_p = where(is_abnormal_number(slow_p), p.data, slow_p)
-                        p.data.copy_(slow_p)  # copy interpolated weights to RAdam param tensor
+                # integrated look ahead...
+                # we do it at the param level instead of group level
+                if state['step'] % group['k'] == 0:
+                    slow_p = state['slow_buffer']  # get access to slow param tensor
+                    slow_p.add_(p.data - slow_p, alpha=self.alpha)  # (fast weights - slow weights) * alpha
+                    if any_abnormal_number(slow_p):
+                        sys.stderr.write('{0} p_data has abnormal value,trident automatically replace these abnormal value to zero.\n'.format(self.__class__.__name__))
+                    slow_p = where(is_abnormal_number(slow_p), p.data, slow_p)
+                    p.data.copy_(slow_p)  # copy interpolated weights to RAdam param tensor
 
         return loss
 
@@ -2035,6 +2047,12 @@ class DiffGrad(Optimizer):
             for p in group['params']:
                 if p.grad is None or not p.requires_grad:
                     continue
+                # cast data type
+                half_precision = False
+                if p.data.dtype == torch.float16:
+                    half_precision = True
+                    p.data = p.data.float()
+                    p.grad = p.grad.float()
                 grad = p.grad.data
                 if grad.is_sparse:
                     raise RuntimeError('diffGrad does not support sparse gradients, please consider SparseAdam instead')
@@ -2053,6 +2071,7 @@ class DiffGrad(Optimizer):
 
                 exp_avg, exp_avg_sq, previous_grad = state['exp_avg'], state['exp_avg_sq'], state['previous_grad']
                 beta1, beta2 = group['betas']
+
                 if self.gradient_centralization in ['all', 'gcc']:
                     if len(list(grad.size())) > 3:
                         grad.add_(-grad.mean(dim=tuple(range(1, grad.dim())), keepdim=True))
@@ -2064,6 +2083,7 @@ class DiffGrad(Optimizer):
                 if group['weight_decay'] != 0:
                     grad.add_(p.data,alpha=group['weight_decay'])
 
+
                 if self.gradient_centralization in ['all', 'gc']:
                     if len(list(grad.size())) > 1:
                         grad.add_(-grad.mean(dim=tuple(range(1, len(list(grad.size())))), keepdim=True))
@@ -2078,7 +2098,6 @@ class DiffGrad(Optimizer):
 
                 diff = abs(previous_grad - grad)
                 dfc = 1. / (1. + torch.exp(-diff))
-                state['previous_grad'] = grad  # used in paper but has the bug that previous grad is overwritten with grad and diff becomes always zero. Fixed in the next line.
 
                 # update momentum with dfc
                 exp_avg1 = exp_avg * dfc
@@ -2086,6 +2105,7 @@ class DiffGrad(Optimizer):
                 step_size = group['lr'] * math.sqrt(bias_correction2) / bias_correction1
 
                 p.data.add_(true_divide(exp_avg1, denom), alpha=-step_size)
+                state[ 'previous_grad'] = grad.copy().detach()  # used in paper but has the bug that previous grad is overwritten with grad and diff becomes always zero. Fixed in the next line.
 
         return loss
 
@@ -2137,6 +2157,12 @@ class Lamb(Optimizer):
             for p in group['params']:
                 if p.grad is None or not p.requires_grad:
                     continue
+                # cast data type
+                half_precision = False
+                if p.data.dtype == torch.float16:
+                    half_precision = True
+                    p.data = p.data.float()
+                    p.grad = p.grad.float()
                 grad = p.grad.data
                 if grad.is_sparse:
                     raise RuntimeError('Lamb does not support sparse gradients, consider SparseAdam instad.')
