@@ -9,7 +9,7 @@ import tensorflow as tf
 from tensorflow.python.framework import ops
 from tensorflow.python.ops import array_ops
 from tensorflow.python.ops import math_ops
-
+from tensorflow.python.ops import nn
 from trident import context
 
 from trident.backend.common import camel2snake, get_class, epsilon, PrintException
@@ -254,6 +254,8 @@ class _ClassificationLoss(Loss):
             output, target = self.flatten_check(output, target)
             loss = self.calculate_loss(*self.preprocess(output, target, **kwargs))
             loss = self._get_reduction(loss)
+            if any_abnormal_number(loss):
+                print('')
             return loss
         except Exception as e:
             print(e)
@@ -507,10 +509,12 @@ class CrossEntropyLoss(_ClassificationLoss):
             if self.is_target_onehot:
                 # -sum([p[i] * log2(q[i]) for i in range(len(p))])
                 if not self.is_logsoftmax:
-                    loss = -reduce_sum(target * log_softmax(output, axis=self.axis, keepdims=True) * sample_weight, axis=-self.axis)
+                    loss = nn.sparse_softmax_cross_entropy_with_logits_v2(labels=argmax(target, -1), logits=output)
+                    # loss = -reduce_sum(target * log_softmax(output, axis=self.axis, keepdims=True) * sample_weight, axis=-self.axis)
 
                 else:
 
+                    # loss = nn.softmax_cross_entropy_with_logits_v2(labels=target, logits=exp(output))
                     loss = -reduce_sum(target * output * sample_weight, axis=self.axis)
 
                 if ndim(loss) > 1:
@@ -522,11 +526,11 @@ class CrossEntropyLoss(_ClassificationLoss):
                 else:
                     return loss
             elif not self.is_target_onehot and ndim(target) == ndim(output) - 1:
-                if not self.is_logsoftmax:
-                    output = log_softmax(output)
+                if self.is_logsoftmax:
+                    output = exp(output)
                 target = cast(target, cast_dtype=dtype.int64)
 
-                loss = tf.nn.sparse_softmax_cross_entropy_with_logits(target, output)
+                loss = nn.sparse_softmax_cross_entropy_with_logits(labels=target, logits=output)
                 return loss
 
                 # loss = reduce_sum(loss, self.axis)
@@ -636,22 +640,22 @@ class F1ScoreLoss(_ClassificationLoss):
         self._built = True
 
     def calculate_loss(self, output, target, **kwargs):
+        with self._name_scope:
+            if self.is_logsoftmax:
+                output = clip(exp(output), 1e-8, 1 - 1e-8)
+                self.from_logits = True
+            if not self.from_logits:
+                output = softmax(output, self.axis)
+            if target.dtype == tf.int64 or self.is_target_onehot == False:
+                target = cast(make_onehot(target, self.num_classes, axis=1), output.dtype)
 
-        if self.is_logsoftmax:
-            output = clip(exp(output), 1e-8, 1 - 1e-8)
-            self.from_logits = True
-        if not self.from_logits:
-            output = softmax(output, self.axis)
-        if target.dtype == tf.int64 or self.is_target_onehot == False:
-            target = cast(make_onehot(target, self.num_classes, axis=1), output.dtype)
-
-        tp = (target * output * self.sample_weight * self.ignore_index_weight).sum(axis=self.axis)
-        # tn = ((1 - target) * (1 - output))
-        # fp = ((1 - target) * output)
-        # fn = (target * (1 - output))
-        precision = true_divide(tp, reduce_sum(output, axis=self.axis))
-        recall = true_divide(tp, reduce_sum(target, axis=self.axis))
-        return 1 - (1 + self.beta ** 2) * precision * recall / (self.beta ** 2 * precision + recall)
+            tp = (target * output * self.sample_weight * self.ignore_index_weight).sum(axis=self.axis)
+            # tn = ((1 - target) * (1 - output))
+            # fp = ((1 - target) * output)
+            # fn = (target * (1 - output))
+            precision = true_divide(tp, reduce_sum(output, axis=self.axis))
+            recall = true_divide(tp, reduce_sum(target, axis=self.axis))
+            return 1 - (1 + self.beta ** 2) * precision * recall / (self.beta ** 2 * precision + recall)
 
 
 class FocalLoss(_ClassificationLoss):
@@ -812,26 +816,26 @@ class DiceLoss(_ClassificationLoss):
         Returns:
 
         """
-        # with self._name_scope:
-        if self.is_logsoftmax:
-            output = exp(output)
-        reduce_axes = list(range(target.ndim))
-        axis = self.axis if self.axis >= 0 else target.ndim + self.axis
-        reduce_axes.remove(0)
+        with self._name_scope:
+            if self.is_logsoftmax:
+                output = exp(output)
+            reduce_axes = list(range(target.ndim))
+            axis = self.axis if self.axis >= 0 else target.ndim + self.axis
+            reduce_axes.remove(0)
 
-        sample_weight = expand_dims(self.sample_weight.to(get_device()) * self.ignore_index_weight.to(get_device()), 0)
-        n_ = ndim(output) - ndim(sample_weight)
+            sample_weight = expand_dims(self.sample_weight.to(get_device()) * self.ignore_index_weight.to(get_device()), 0)
+            n_ = ndim(output) - ndim(sample_weight)
 
-        sample_weight = cast(expand_dims(self.sample_weight * self.ignore_index_weight, 0), output.dtype)
-        n_ = ndim(output) - ndim(sample_weight)
-        for n in range(n_):
-            sample_weight = expand_dims(sample_weight, -1)
-
-        intersection = reduce_sum(target * output * sample_weight, axis=reduce_axes)
-        den1 = reduce_sum(output * sample_weight, axis=reduce_axes)
-        den2 = reduce_sum(target * sample_weight, axis=reduce_axes)
-        dice = 1.0 - (2.0 * intersection + self.smooth) / (den1 + den2 + self.smooth)
-        return dice
+            sample_weight = cast(expand_dims(self.sample_weight * self.ignore_index_weight, 0), output.dtype).detach()
+            n_ = ndim(output) - ndim(sample_weight)
+            for n in range(n_):
+                sample_weight = expand_dims(sample_weight, 0)
+            target = target.detach()
+            intersection = reduce_sum(target * output * sample_weight, axis=reduce_axes)
+            den1 = reduce_sum(output * sample_weight, axis=reduce_axes)
+            den2 = reduce_sum(target * sample_weight, axis=reduce_axes)
+            dice = 1.0 - (2.0 * intersection + self.smooth) / (den1 + den2 + self.smooth)
+            return dice
 
 
 class L1Loss(_PairwiseLoss):
