@@ -5,6 +5,7 @@ import numpy as np
 import math
 import itertools
 import copy
+from  typing import Union,Optional,List,Tuple
 import tensorflow as tf
 from tensorflow.python.framework import tensor_shape
 from tensorflow.python.ops import nn_ops
@@ -33,7 +34,7 @@ def flatten_check(output, target):
         raise ValueError('output and target have diffent elements.')
 
 
-def accuracy(output, target, topk=1, axis=-1,ignore_index=-100, exclude_mask=False):
+def accuracy(output:Tensor, target:Tensor, topk:int=1, axis:int=-1,ignore_index:Union[int,list]=-100, exclude_mask:bool=False):
     """Computes the precision@k for the specified values of k
     prec1, prec5 = accuracy(output.data, target, topk=(1, 5))
     """
