@@ -1,10 +1,9 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
-import numpy as np
 import tensorflow as tf
 
-from trident.backend.common import get_session, get_function, camel2snake, snake2camel
+from trident.backend.common import get_session, get_function
 from  trident.backend.tensorflow_backend import Layer
 from  trident.backend.tensorflow_ops import *
 __all__ = ['l1_reg','l2_reg','orth_reg','get_reg','total_variation_norm_reg']
@@ -14,7 +13,7 @@ _session=get_session()
 _epsilon=_session.epsilon
 
 
-def l1_reg(model:Layer,reg_weight=1e-4):
+def l1_reg(model:Layer,reg_weight=1e-6):
     loss=0.0
     for name, param in model.named_parameters():
         if  'bias' not in name and not any_abnormal_number(param) and param.trainable:
@@ -22,7 +21,7 @@ def l1_reg(model:Layer,reg_weight=1e-4):
     return loss
 
 
-def l2_reg(model:Layer ,reg_weight=1e-4):
+def l2_reg(model:Layer ,reg_weight=1e-6):
     loss = 0.0
     for name, param in model.named_parameters():
         if  'bias' not in name and not any_abnormal_number(param) and param.trainable:
