@@ -30,7 +30,7 @@ from trident.callbacks.visualization_callbacks import *
 from trident.data.data_provider import *
 from trident.misc.ipython_utils import *
 from trident.misc.visualization_utils import tile_rgb_images, loss_metric_curve
-from trident.backend.tensorspec import TensorSpec, assert_spec_compatibility,get_signature
+from trident.backend.tensorspec import TensorSpec, assert_spec_compatibility, get_signature
 from trident.loggers.history import HistoryBase
 
 __all__ = ['TrainingPlan', 'GanTrainingPlan']
@@ -232,39 +232,11 @@ class TrainingPlan(object):
         self.num_epochs = num_epochs
         return self
 
-    @deprecated('0.7.0', 'with_batch_size')
-    def within_minibatch_size(self, minibatch_size: int):
-        self._batch_size = minibatch_size
-        return self
 
     def with_batch_size(self, batch_size: int):
         self._batch_size = batch_size
         return self
 
-    @deprecated('0.7.0', 'with_tensorboard')
-    def within_tensorboard(self):
-        make_dir_if_need(os.path.join(working_directory, 'Logs'))
-        # check weather have tensorboard
-        if get_backend() == 'pytorch':
-            try:
-                from trident.loggers.pytorch_tensorboard import SummaryWriter
-                ctx.try_enable_tensorboard(SummaryWriter(os.path.join(working_directory, 'Logs')))
-
-            except Exception as e:
-                print('Tensorboard initialize failed, please check the installation status about Tensorboard.')
-                print(e)
-                PrintException()
-        elif get_backend() == 'tensorflow':
-            try:
-                from trident.loggers.tensorflow_tensorboard import SummaryWriter
-                ctx.try_enable_tensorboard(SummaryWriter(os.path.join(working_directory, 'Logs')))
-
-            except Exception as e:
-                print('Tensorboard initialize failed, please check the installation status about Tensorboard.')
-                print(e)
-                PrintException()
-        return self
-
     def with_tensorboard(self):
         make_dir_if_need(os.path.join(working_directory, 'Logs'))
         # check weather have tensorboard
@@ -315,7 +287,7 @@ class TrainingPlan(object):
 
     def print_gradients_scheduling(self, frequency: int, unit='batch'):
 
-        pg = PrintGradientsCallback(frequency=frequency,unit=unit,)
+        pg = PrintGradientsCallback(frequency=frequency, unit=unit, )
         pg.is_shared = False
         self.callbacks.append(pg)
         return self
@@ -338,7 +310,7 @@ class TrainingPlan(object):
         if unit not in ['batch', 'epoch']:
             raise ValueError('unit should be batch or epoch')
 
-        tile = TileImageCallback(frequency=frequency,unit=unit,
+        tile = TileImageCallback(frequency=frequency, unit=unit,
                                  save_path=save_path, name_prefix=name_prefix, include_input=include_input,
                                  include_output=include_output, include_target=include_target,
                                  include_mask=include_mask, imshow=imshow)
@@ -362,7 +334,7 @@ class TrainingPlan(object):
                 except Exception as e:
                     PrintException()
                     raise ValueError('save_path:{0} is not valid path'.format(folder))
-        plot = PlotLossMetricsCallback(frequency=frequency,unit=unit,
+        plot = PlotLossMetricsCallback(frequency=frequency, unit=unit,
                                        save_path=save_path, name_prefix=name_prefix.format(get_time_suffix()),
                                        clean_ipython_output_frequency=clean_ipython_output_frequency, imshow=imshow)
         self.callbacks.append(plot)
@@ -386,8 +358,8 @@ class TrainingPlan(object):
             existing_data_feed = None
             if 'data_feed' in trainingitem.training_context:
                 existing_data_feed = trainingitem.training_context['data_feed']
-            if not hasattr(trainingitem._model,'_signature') or  trainingitem._model._signature is None:
-                trainingitem._model._signature=get_signature(trainingitem._model)
+            if not hasattr(trainingitem._model, '_signature') or trainingitem._model._signature is None:
+                trainingitem._model._signature = get_signature(trainingitem._model)
 
             data_feed = OrderedDict()
             # datasets = data_provider.traindata.get_datasets()
@@ -405,16 +377,16 @@ class TrainingPlan(object):
                 for inp in v.signature.inputs.key_list:
                     data_feed[inp] = None
 
-            if 'x' in data_feed :
-                if len(data_symbols)==1 and  data_symbols[0] in available_items:
+            if 'x' in data_feed:
+                if len(data_symbols) == 1 and data_symbols[0] in available_items:
                     data_feed['x'] = data_symbols[0]
-                    available_items.remove(data_symbols[0] )
+                    available_items.remove(data_symbols[0])
 
                 if 'x' in available_items:
-                    data_feed['x'] ='x'
+                    data_feed['x'] = 'x'
                     available_items.remove('x')
                 elif 'input' in available_items:
-                    data_feed['x'] ='input'
+                    data_feed['x'] = 'input'
                     available_items.remove('input')
 
             if len(trainingitem.signature.inputs) == len(data_symbols) == 1:
@@ -457,7 +429,6 @@ class TrainingPlan(object):
         for item in self.training_items.value_list:
             item.train()
 
-
     def do_on_training_end(self):
         for callback in self.callbacks:
             callback.on_training_end(self.__dict__)
@@ -559,7 +530,7 @@ class TrainingPlan(object):
                                 need_out_sample_evaluation = True
 
                             iter_testdata = None
-                            if isinstance(data_provider, (DataProvider,TextSequenceDataProvider)) and data_provider.testdata is not None and need_out_sample_evaluation:
+                            if isinstance(data_provider, (DataProvider, TextSequenceDataProvider)) and data_provider.testdata is not None and need_out_sample_evaluation:
                                 return_test = data_provider.next_test()
                                 if return_test is not None:
                                     iter_testdata = OrderedDict()
@@ -578,18 +549,18 @@ class TrainingPlan(object):
                                 trainitem.training_context['data_template'] = data_provider.traindata.data_template
                                 trainitem.training_context['collect_data_inteval'] = collect_data_inteval
                                 trainitem.training_context['model_name'] = trainitem_name
-                                start_epoch=0 if not hasattr(trainitem,'start_epoch') else trainitem.start_epoch
-                                if epoch <start_epoch:
+                                start_epoch = 0 if not hasattr(trainitem, 'start_epoch') else trainitem.start_epoch
+                                if epoch < start_epoch:
                                     trainitem.training_context['stop_update'] = 1
-                                num_epoch=self.num_epochs if only_steps == False else 1
-                                num_batches=len(data_provider.batch_sampler) if only_steps == False else max_batches
-                                current_batch=mbs if only_steps == False else self.steps
-                                current_epoch=epoch if only_steps == False else 0
-                                if current_batch ==0:
+                                num_epoch = self.num_epochs if only_steps == False else 1
+                                num_batches = len(data_provider.batch_sampler) if only_steps == False else max_batches
+                                current_batch = mbs if only_steps == False else self.steps
+                                current_epoch = epoch if only_steps == False else 0
+                                if current_batch == 0:
                                     trainitem.do_on_epoch_start()
-                                trainitem.steps=self.steps
-                                trainitem.current_epoch=current_epoch
-                                trainitem.current_batch=current_batch
+                                trainitem.steps = self.steps
+                                trainitem.current_epoch = current_epoch
+                                trainitem.current_batch = current_batch
                                 trainitem.train_model(train_data, test_data,
                                                       current_epoch,
                                                       current_batch,
@@ -600,8 +571,9 @@ class TrainingPlan(object):
                                                       is_print_batch_progress=self.print_progress_unit == 'batch' and mbs > 0 and mbs % self.print_progress_frequency == 0,
                                                       is_print_epoch_progress=self.print_progress_unit == 'epoch' and epoch > 0 and epoch % self.print_progress_frequency == 0,
                                                       log_gradients=keep_gradient_history, log_weights=keep_weights_history,
-                                                      accumulate_grads=(trainitem.training_context['steps']+1) % trainitem.accumulation_steps != 0, is_out_sample_evaluation=need_out_sample_evaluation)
-                                if current_batch==num_batches-1:
+                                                      accumulate_grads=(trainitem.training_context['steps'] + 1) % trainitem.accumulation_steps != 0,
+                                                      is_out_sample_evaluation=need_out_sample_evaluation)
+                                if current_batch == num_batches - 1:
                                     trainitem.do_on_epoch_end()
                             self.steps += 1
 
@@ -619,7 +591,7 @@ class TrainingPlan(object):
                                             compare_dict[k] = OrderedDict()
                                         compare_dict[k][k + "/" + trainitem_name] = v[-1][1]
                                 for k, v in compare_dict.items():
-                                    self.summary_writer.add_scalars(k, v, step)
+                                    ctx.summary_writer.add_scalars(k, v, step)
 
                             if (self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0) or \
                                     (self.print_progress_unit == 'epoch' and (epoch + 1) % self.print_progress_frequency == 0):
@@ -1299,7 +1271,7 @@ class GanTrainingPlan(TrainingPlan):
                                             compare_dict[k] = OrderedDict()
                                         compare_dict[k][k + "/" + trainitem_name] = v[-1][1]
                                 for k, v in compare_dict.items():
-                                    self.summary_writer.add_scalars(k, v, step)
+                                    ctx.summary_writer.add_scalars(k, v, step)
 
                             if (self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0) or \
                                     (self.print_progress_unit == 'epoch' and (epoch + 1) % self.print_progress_frequency == 0):
