@@ -25,19 +25,19 @@ from gym.wrappers import LazyFrames
 from matplotlib import animation
 from pyvirtualdisplay import Display
 
-from trident.backend.common import make_dir_if_need,get_plateform,get_time_suffix,sanitize_path,if_none
+from trident.backend.common import make_dir_if_need, get_plateform, get_time_suffix, sanitize_path, if_none
 from trident.misc.ipython_utils import *
-from trident.reinforcement.utils import ObservationType,ActionStrategy
+from trident.reinforcement.utils import ObservationType, ActionStrategy
 from trident import context
 from trident.data.vision_transforms import Resize
-from trident.backend.pillow_backend import array2image,image2array
+from trident.backend.pillow_backend import array2image, image2array
 
+ctx = context._context()
+__all__ = ['NoopResetEnv', 'EpisodicLifeEnv', 'MaxAndSkipEnv', 'RunningAvgAndSkipEnv', 'WarpFrame', 'FrameStack', 'TimeAwareObservation', 'VideoRecording']
 
-ctx=context._context()
-__all__ = ['NoopResetEnv','EpisodicLifeEnv','MaxAndSkipEnv','RunningAvgAndSkipEnv','WarpFrame','FrameStack','TimeAwareObservation','VideoRecording']
 
 class NoopResetEnv(gym.Wrapper):
-    def __init__(self, env, noop_max=30,preferred_actions=None):
+    def __init__(self, env, noop_max=30, preferred_actions=None):
         """Sample initial states by taking random number of no-ops on reset.
         No-op is assumed to be action 0.
         """
@@ -45,9 +45,8 @@ class NoopResetEnv(gym.Wrapper):
         self.noop_max = noop_max
         self.override_num_noops = None
         self.noop_action = 0
-        self.preferred_actions=preferred_actions
-        #assert env.unwrapped.get_action_meanings()[0] == 'NOOP'
-
+        self.preferred_actions = preferred_actions
+        # assert env.unwrapped.get_action_meanings()[0] == 'NOOP'
 
     def reset(self, **kwargs):
         """ Do no-op action for a number of steps in [1, noop_max]."""
@@ -59,14 +58,14 @@ class NoopResetEnv(gym.Wrapper):
         assert noops > 0
 
         obs = None
-        remain_noops=noops
-        while remain_noops>0:
-            if random.random()<0.8:
-                if isinstance(self.preferred_actions,list) and len(self.preferred_actions)>0:
+        remain_noops = noops
+        while remain_noops > 0:
+            if random.random() < 0.8:
+                if isinstance(self.preferred_actions, list) and len(self.preferred_actions) > 0:
                     obs, _, done, _ = self.env.step(random.choice(self.preferred_actions))
                 else:
                     obs, _, done, _ = self.env.step(self.noop_action)
-                remain_noops-=1
+                remain_noops -= 1
                 if done:
                     obs = self.env.reset(**kwargs)
         return obs
@@ -78,18 +77,17 @@ class EpisodicLifeEnv(gym.Wrapper):
         Done by DeepMind for the DQN and co. since it helps value estimation.
         """
         gym.Wrapper.__init__(self, env)
-        self.lives =  self.env.unwrapped._life
-        self.was_real_done  = True
+        self.lives = self.env.unwrapped._life
+        self.was_real_done = True
 
     def step(self, action):
         obs, reward, done, info = self.env.step(action)
-        self.state=obs
         self.was_real_done = done
         # check current lives, make loss of life terminal,
         # then update lives to handle bonus lives
         lives = self.env.unwrapped._life
-        #lives = self.env.unwrapped.ale.lives()
-        if self.lives > lives > 0 :
+        # lives = self.env.unwrapped.ale.lives()
+        if self.lives > lives > 0:
             # for Qbert sometimes we stay in lives == 0 condition for a few frames
             # so it's important to keep lives > 0, so that we only reset once
             # the environment advertises done.
@@ -104,12 +102,12 @@ class EpisodicLifeEnv(gym.Wrapper):
         """
         if self.was_real_done:
             obs = self.env.reset(**kwargs)
-            self.state=obs
+
         else:
             # no-op step to advance from terminal/lost life state
             obs, _, _, _ = self.env.step(0)
-            self.state = obs
-        #self.lives = self.env.unwrapped.ale.lives()
+
+        # self.lives = self.env.unwrapped.ale.lives()
         self.lives = self.env.unwrapped._life
         return obs
 
@@ -119,8 +117,8 @@ class MaxAndSkipEnv(gym.Wrapper):
         """Return only every `skip`-th frame"""
         gym.Wrapper.__init__(self, env)
         # most recent raw observations (for max pooling across time steps)
-        self._obs_buffer = np.zeros((2,)+env.observation_space.shape, dtype=np.uint8)
-        self._skip       = skip
+        self._obs_buffer = np.zeros((2,) + env.observation_space.shape, dtype=np.uint8)
+        self._skip = skip
 
     def step(self, action):
         """Repeat action, sum reward, and max over last observations."""
@@ -128,12 +126,12 @@ class MaxAndSkipEnv(gym.Wrapper):
         done = None
         for i in range(self._skip):
             obs, reward, done, info = self.env.step(action)
-           
+
             if i == self._skip - 2: self._obs_buffer[0] = obs
             if i == self._skip - 1: self._obs_buffer[1] = obs
             total_reward += reward
-            if done or  self.env.unwrapped._is_dying or self.env.unwrapped._is_dead or self.env.unwrapped._life < 2:
-                done=True
+            if done or self.env.unwrapped._is_dying or self.env.unwrapped._is_dead or self.env.unwrapped._life < 2:
+                done = True
                 break
         # Note that the observation on the done=True frame
         # doesn't matter
@@ -168,8 +166,8 @@ class RunningAvgAndSkipEnv(gym.Wrapper):
                 break
         # Note that the observation on the done=True frame
         # doesn't matter
-        weight=np.reshape(np.array([1,2,3,4]),(4,1,1,1))
-        max_frame = (weight*self._obs_buffer).sum(axis=0)/10.0
+        weight = np.reshape(np.array([1, 2, 3, 4]), (4, 1, 1, 1))
+        max_frame = (weight * self._obs_buffer).sum(axis=0) / 10.0
 
         return max_frame, total_reward, done, info
 
@@ -217,7 +215,7 @@ class WarpFrame(gym.ObservationWrapper):
 
         if self._grayscale:
             frame = cv2.cvtColor(frame, cv2.COLOR_RGB2GRAY)
-        frame = cv2.resize( frame[34:194], (self._width, self._height), interpolation=cv2.INTER_AREA )
+        frame = cv2.resize(frame[34:194], (self._width, self._height), interpolation=cv2.INTER_AREA)
         if self._grayscale:
             frame = np.expand_dims(frame, -1)
 
@@ -260,48 +258,59 @@ class FrameStack(gym.Wrapper):
         assert len(self.frames) == self.k
         return LazyFrames(list(self.frames))
 
+
 class TimeAwareObservation(gym.ObservationWrapper):
     r"""Augment the observation with current time step in the trajectory.
     .. note::
         Currently it only works with one-dimensional observation space. It doesn't
         support pixel observation space yet.
     """
+
     def __init__(self, env, max_time=None):
         super(TimeAwareObservation, self).__init__(env)
-        self.max_time=max_time
-        if  isinstance(env.observation_space, Box) and env.observation_space.dtype == np.float32:
+        self.max_time = max_time
+        if isinstance(env.observation_space, Box) and env.observation_space.dtype == np.float32:
             low = np.append(self.observation_space.low, 0.0)
             high = np.append(self.observation_space.high, np.inf)
             self.observation_space = Box(low, high, dtype=np.float32)
-            self.obsetvation_type=ObservationType.Box
-        elif  isinstance(env.observation_space, Box)  and ((env.observation_space.low.max()==0 and env.observation_space.high.min()==255 )or env.observation_space.dtype == np.uint8):
+            self.obsetvation_type = ObservationType.Box
+        elif isinstance(env.observation_space, Box) and (
+                (env.observation_space.low.max() == 0 and env.observation_space.high.min() == 255) or env.observation_space.dtype == np.uint8):
             self.obsetvation_type = ObservationType.Image
 
-
     def observation(self, observation):
-        observation=observation.copy()
-        if self.obsetvation_type==ObservationType.Box:
-            return np.append(observation, self.t if self.max_time is None or self.max_time==0 else self.t/float(self.max_time ))
-        elif self.obsetvation_type==ObservationType.Image and self.max_time>0:
-            ratio = self.t / float(self.max_time)
-            if hasattr(super(TimeAwareObservation, self).unwrapped,'_time'):
-                ratio =(super(TimeAwareObservation, self).unwrapped._time/ float(self.max_time))
 
+        if self.obsetvation_type == ObservationType.Box:
+            return np.append(observation, self.t if self.max_time is None or self.max_time == 0 else self.t / float(self.max_time))
+        elif self.obsetvation_type == ObservationType.Image and self.max_time > 0:
+            ratio = self.t / float(self.max_time)
+            if hasattr(super(TimeAwareObservation, self).unwrapped, '_time'):
+                ratio = (super(TimeAwareObservation, self).unwrapped._time / float(self.max_time))
+            original_dtype = observation.dtype
             pixels = builtins.round(self.observation_space.shape[1] * ratio)
+            H, W, C = observation.shape
+            _observation = observation.copy().astype(np.float32)
+            obj_base = np.zeros((W, W, C)).astype(np.float32)
+            obj_base[:H, :, :] = _observation
+            noise_mask = np.clip(np.expand_dims(np.expand_dims(np.linspace(0, 2, W), 0), -1), 0, 1).astype(np.float32)
+            obj_base = obj_base * noise_mask
+            obj_base = np.clip(obj_base + (1 - noise_mask) * (0.5 + np.random.standard_normal((W, W, C))), 0, 255)
+
             if len(self.observation_space.shape) == 2:
-                observation[-10:, :] = 0
-                observation[-10:, -pixels:] = 255
+                obj_base[H - W:, :] = 0
+                obj_base[H - W:, -pixels:] = 255
             elif len(self.observation_space.shape) == 3:
-                observation[-10:, :, :] = 0
-                observation[-10:, -pixels:, :] = 255
-            self.state=observation
-            return observation
+                obj_base[H - W:, :, :] = 0
+                obj_base[H - W:, -pixels:, :] = 255
+            return np.clip(obj_base, 0, 255).astype(original_dtype)
 
     def step(self, action):
-        return  self.env.step(action)
+        observation, reward, done, info = self.env.step(action)
+        time_awareness_obj = self.observation(observation)
+        return time_awareness_obj, reward, done, info
 
     def reset(self, **kwargs):
-        self.t=0
+        self.t = 0
         return self.env.reset(**kwargs)
 
     # def render(self, mode='human', **kwargs):
@@ -312,24 +321,26 @@ class TimeAwareObservation(gym.ObservationWrapper):
     #     elif mode=='rgb_array':
     #         return super(TimeAwareObservation, self).render('rgb_array')
 
+
 class _VirtualDisplaySingleton(object):
-    def __new__(cls,*args,**kwargs):
-        if not hasattr(cls,"_instance"):
+    def __new__(cls, *args, **kwargs):
+        if not hasattr(cls, "_instance"):
             cls._instance = super().__new__(cls)
         return cls._instance
 
-    def __init__(self,size=(1024, 768)):
+    def __init__(self, size=(1024, 768)):
         self.size = size
 
-        if not hasattr(self,"_display"):
-            self._display = Display(visible=0,size=self.size)
+        if not hasattr(self, "_display"):
+            self._display = Display(visible=0, size=self.size)
             original = subprocess.Popen
-            def Popen(cmd,pass_fds,stdout,stderr,shell):
-                return original(cmd,pass_fds=pass_fds,
-                                stdout=stdout,stderr=stderr,
-                                shell=shell,preexec_fn=os.setpgrp)
 
-            with patch("subprocess.Popen",Popen):
+            def Popen(cmd, pass_fds, stdout, stderr, shell):
+                return original(cmd, pass_fds=pass_fds,
+                                stdout=stdout, stderr=stderr,
+                                shell=shell, preexec_fn=os.setpgrp)
+
+            with patch("subprocess.Popen", Popen):
                 self._display.start()
 
     def _restart_display(self):
@@ -341,7 +352,8 @@ class VirtualDisplay(Wrapper):
     """
     Wrapper for running Xvfb
     """
-    def __init__(self,env,size=(1024, 768)):
+
+    def __init__(self, env, size=(1024, 768)):
         """
         Wrapping environment and start Xvfb
         """
@@ -349,11 +361,11 @@ class VirtualDisplay(Wrapper):
         self.size = size
         self._display = _VirtualDisplaySingleton(size)
 
-    def render(self,mode=None,**kwargs):
+    def render(self, mode=None, **kwargs):
         """
         Render environment
         """
-        return self.env.render(mode='rgb_array',**kwargs)
+        return self.env.render(mode='rgb_array', **kwargs)
 
 
 class VideoRecording(gym.Wrapper):
@@ -367,7 +379,8 @@ class VideoRecording(gym.Wrapper):
     --------
     gym.wrappers.Monitor : https://github.com/openai/gym/blob/master/gym/wrappers/monitor.py
     """
-    def __init__(self,env,directory: Optional[str]=None,enabled=False,fps=None,min_frames=None,done_then_finish=True,name_prefix=None,**kwargs):
+
+    def __init__(self, env, directory: Optional[str] = None, enabled=False, fps=None, min_frames=None, done_then_finish=True, name_prefix=None, **kwargs):
         """
         Initialize Monitor class
 
@@ -379,70 +392,69 @@ class VideoRecording(gym.Wrapper):
         """
         gym.Wrapper.__init__(self, env)
         self._recording_enabled = enabled
-        self._is_recording=False
-        self.num_frames=0
-        self.directory =directory
-        self.done_then_finish=done_then_finish
+        self._is_recording = False
+        self.num_frames = 0
+        self.directory = directory
+        self.done_then_finish = done_then_finish
         if directory is None:
             self.directory = make_dir_if_need('videos')
         else:
-            self.directory=make_dir_if_need(sanitize_path(directory))
+            self.directory = make_dir_if_need(sanitize_path(directory))
         self.fps = fps
         if 'video.frames_per_second' not in self.env.metadata:
-            self.env.metadata['video.frames_per_second']=self.fps if self.fps is not None else 30
-        self.min_frames=min_frames
+            self.env.metadata['video.frames_per_second'] = self.fps if self.fps is not None else 30
+        self.min_frames = min_frames
         self.videos = []
-        self._display=None
+        self._display = None
         shp = env.observation_space.shape
-        if get_plateform()!='windows':
-            self._display = _VirtualDisplaySingleton((shp[1],shp[0]))
+        if get_plateform() != 'windows':
+            self._display = _VirtualDisplaySingleton((shp[1], shp[0]))
         if name_prefix is None:
-            name_prefix='video'
+            name_prefix = 'video'
 
-        self.name_prefix =name_prefix
-        self.current_recording_path=None
+        self.name_prefix = name_prefix
+        self.current_recording_path = None
         self.vw = None
-        self.resize=Resize((84,84),False,cv2.INTER_LANCZOS4)
-        #self.vw2 = None
-        self.frame_steps=0
-        self.prev_screen=None
+        self.resize = Resize((84, 84), False, cv2.INTER_LANCZOS4)
+        # self.vw2 = None
+        self.frame_steps = 0
+        self.prev_screen = None
         self.current_screen = None
-        self.prev_observation=None
-        self.current_observation=None
+        self.prev_observation = None
+        self.current_observation = None
         self.prev_reward = 0
         self.prev_done = False
         self.info = None
-        self.frame_stats=[]
-
-
-
+        self.frame_stats = []
 
     def create_video_writer(self):
         shp = super().render(mode='rgb_array').shape
         video_name = os.path.join(self.directory, self.name_prefix + '_' + get_time_suffix() + '.avi')
-        #video_name2 = os.path.join(self.directory, self.name_prefix + '_obj_' + get_time_suffix() + '.avi')
+        # video_name2 = os.path.join(self.directory, self.name_prefix + '_obj_' + get_time_suffix() + '.avi')
         self.current_recording_path = video_name
+        # self.current_recording_path2 = video_name2
 
-        self.vw = cv2.VideoWriter(video_name, cv2.VideoWriter_fourcc('M', 'J', 'P', 'G'), if_none(self.fps,self.env.metadata['video.frames_per_second']), (shp[1], shp[0]))
+        self.vw = cv2.VideoWriter(video_name, cv2.VideoWriter_fourcc('M', 'J', 'P', 'G'), if_none(self.fps, self.env.metadata['video.frames_per_second']), (shp[1], shp[0]))
 
-        #self.vw2 = cv2.VideoWriter(video_name2, cv2.VideoWriter_fourcc('M', 'J', 'P', 'G'), self.env.metadata['video.frames_per_second'], (84*4, 84*4))
+        # self.vw2 = cv2.VideoWriter(video_name2, cv2.VideoWriter_fourcc('M', 'J', 'P', 'G'), self.env.metadata['video.frames_per_second'], (84*4, 84*4))
 
         self._is_recording = True
-        self.frame_steps  = 0
+        self.frame_steps = 0
 
     def close_video_recorder(self):
 
-        if self._is_recording   and (self.vw is not None and self.frame_steps >0):
+        if self._is_recording and (self.vw is not None and self.frame_steps > 0):
             self.vw.release()
-            self.vw=None
-            #self.vw2.release()
+            self.vw = None
+            # self.vw2.release()
             self._is_recording = False
-            if self.min_frames is not None and self.frame_steps <self.min_frames:
+            if self.min_frames is not None and self.frame_steps < self.min_frames:
                 os.remove(self.current_recording_path)
+                #   os.remove(self.current_recording_path2 )
             else:
                 self.videos.append(self.current_recording_path)
-        
-    def step(self,action):
+
+    def step(self, action):
         """
         Step Environment
         """
@@ -450,28 +462,17 @@ class VideoRecording(gym.Wrapper):
         try:
 
             observation, reward, done, info = self.env.step(action)
-            self.done=done
-            state=observation
+            self.done = done
+
             new_render = self.render('rgb_array').copy()
 
-            # fig = self.env.cube.render(flat=True)
-            # fig.canvas.draw()
-            #
-            # w, h = fig.canvas.get_width_height()
-            # buf = np.fromstring(fig.canvas.tostring_rgb(), dtype=np.uint8).reshape((h, w, 3))
-            # #new_render = cv2.cvtColor(buf.reshape((h, w, 3)), cv2.COLOR_RGB2BGR)
-            # new_render = buf.copy()
-            # plt.imshow(array2image( buf.copy()), interpolation="nearest", animated=True)
-            # plt.ioff()
-            # plt.show()
-            # plt.clf()
-            #plt.close("all")
+            self.prev_screen = copy.deepcopy(self.current_screen)
+            self.current_screen = new_render
+            self.prev_observation = copy.deepcopy(self.current_observation)
+            self.current_observation = observation.copy()
 
-
-            # self.prev_screen = copy.deepcopy(self.current_screen)
-            # self.current_screen = new_render
-            # self.prev_observation = copy.deepcopy(self.current_observation)
-            # self.current_observation = state
+            if self.current_observation.shape != self.prev_observation.shape:
+                self.prev_observation = self.current_observation.copy()
 
             if self.vw is None:
                 self.create_video_writer()
@@ -480,58 +481,50 @@ class VideoRecording(gym.Wrapper):
 
             else:
                 if self._recording_enabled and self._is_recording:
-
                     self.vw.write(cv2.cvtColor(new_render, cv2.COLOR_RGB2BGR))
                     self.frame_steps += 1
                     self.env.unwrapped.frame_steps = self.frame_steps
-
-                    # obj_frame= cv2.resize(cv2.cvtColor(observation.copy(), cv2.COLOR_RGB2BGR),(84*4,84*4))
-
-                    #self.vw2.write(obj_frame)
-
+                    # obj_frame= cv2.resize(cv2.cvtColor(observation.copy().copy(), cv2.COLOR_RGB2BGR),(84*4,84*4),cv2.INTER_LANCZOS4)
+                    # self.vw2.write(obj_frame)
 
             self.prev_reward = reward
             self.prev_done = done
             self.info = info
-            self.state=state
 
-            return observation, reward, done, info
+            self.state = self.resize(observation)
+            return self.state.copy(), reward, done, info
         except KeyboardInterrupt as k:
             self.close_video_recorder()
             raise
 
-
-    def reset(self,**kwargs):
+    def reset(self, **kwargs):
         """
         Reset Environment
         """
         try:
 
-            if self.done_then_finish and self._is_recording and self.frame_steps >0:
+            if self.done_then_finish and self._is_recording and self.frame_steps > 0:
                 self.close_video_recorder()
-            if self._recording_enabled and (not self._is_recording or  self.vw is None ):
+            if self._recording_enabled and (not self._is_recording or self.vw is None):
                 self.create_video_writer()
 
-            self.state=self.resize(self.env.reset())#self.resize(super().reset(**kwargs))
-            #self.prev_screen = self.render('rgb_array').copy()
-            self.prev_observation = copy.deepcopy(self.state)
-
-            self.prev_observation =  copy.deepcopy(self.state)
-            self.current_observation =  copy.deepcopy(self.state)
             self.frame_stats = []
-            self.prev_reward=0
-            self.prev_done=False
-            self.info=None
+            self.prev_reward = 0
+            self.prev_done = False
+            self.info = None
             self.frame_steps = 0
-            self.env.unwrapped.frame_steps=self.frame_steps
-            
+            self.env.unwrapped.frame_steps = self.frame_steps
+            observation = self.env.reset()
+            self.prev_observation = observation
+            self.current_observation = observation
+            self.state = self.resize(observation)
 
-            return  self.state
+            return self.resize(observation)
         except KeyboardInterrupt:
             self.close_video_recorder()
             raise
 
-    def display(self,reset: bool=False):
+    def display(self, reset: bool = False):
         """
         Display saved all movies
 
