@@ -7,7 +7,7 @@ import time
 import uuid
 from copy import deepcopy
 from itertools import count
-from typing import List, Tuple, Optional, Union, Callable, Any, Iterable, Mapping, TypeVar
+from typing import List, Tuple, Optional, Union, Callable, Any, Iterable, Mapping, TypeVar,Dict
 import numpy as np
 import torch
 from torch.distributions import Bernoulli, Categorical
@@ -25,7 +25,7 @@ import_or_install('gym')
 import gym
 
 _session = get_session()
-__all__ = ['PolicyBase', 'ActorCriticPolicy', 'DqnPolicy', 'PGPolicy' 'A2CPolicy', 'PPOPolicy']
+__all__ = ['PolicyBase', 'ActorCriticPolicy', 'DqnPolicy', 'PGPolicy', 'A2CPolicy', 'PPOPolicy']
 
 
 class PolicyBase(Model):
@@ -465,8 +465,13 @@ class ActorCriticPolicy(MuiltiNetwork):
         self.replay_unit = replay_unit
         self.name = name
         self.action_strategy = action_strategy
-        self.add_network('actor', output=actor)
-        self.add_network('critic', output=critic)
+        self.state_shape=state_shape
+        if self.state_shape is not None:
+            self.add_network('actor', input_shape=self.state_shape, output=actor)
+            self.add_network('critic', input_shape=self.state_shape, output=critic)
+        else:
+            self.add_network('actor',inputs=self.get_observation(), output=actor)
+            self.add_network('critic',inputs=self.get_observation(), output=critic)
         self.setting_network()
         self.train_steps = -1
         self.frame_steps = -1
@@ -484,8 +489,8 @@ class ActorCriticPolicy(MuiltiNetwork):
         self._networks['actor'].training_context['skip_generate_output'] = True
 
     def get_observation(self):
-        if hasattr(self.env, 'state'):
-            return expand_dims(self.data_preprocess(self.env.state), 0)
+        if hasattr(self.self, 'state'):
+            return expand_dims(self.data_preprocess(self.state), 0)
         elif hasattr(self.env, 'screen'):
             return expand_dims(self.data_preprocess(self.env.screen), 0)
         elif 'observation' in self.env.metadata['render.modes']:
@@ -596,7 +601,7 @@ class ActorCriticPolicy(MuiltiNetwork):
         else:
             raise ValueError('target_model and trained_model should be trident Model or Layer.')
 
-    def calculate_gae(self, values, next_values, rewards, dones, gamma: float, gae_lambda: float):
+    def calculate_gae(self, values, next_values, rewards, dones, gamma: float, gae_lambda: float,**kwargs):
         try:
             R = [0] * len(rewards)
             delta = to_tensor(rewards) + next_values * gamma - values
@@ -610,7 +615,7 @@ class ActorCriticPolicy(MuiltiNetwork):
         except Exception as e:
             print(e)
 
-    def update_actor(self, current_episode=0, current_step=0, num_episodes=100, repeat_train=1, train_timing=None, done=False, batch_size=1, accumulate_grads=False):
+    def update_actor(self, current_episode=0, current_step=0, num_episodes=100, repeat_train=1, train_timing=None, done=False, batch_size=1, accumulate_grads=False,**kwargs):
         is_collect_data = False
         for i in range(repeat_train):
             self.actor.train_model(None, None,
@@ -624,7 +629,7 @@ class ActorCriticPolicy(MuiltiNetwork):
                                    log_gradients=False, log_weights=False,
                                    accumulate_grads=accumulate_grads)
 
-    def update_critic(self, current_episode=0, current_step=0, num_episodes=100, repeat_train=1, train_timing=None, done=False, batch_size=1, accumulate_grads=False):
+    def update_critic(self, current_episode=0, current_step=0, num_episodes=100, repeat_train=1, train_timing=None, done=False, batch_size=1, accumulate_grads=False,**kwargs):
         is_collect_data = False
         for i in range(repeat_train):
             self.critic.train_model(None, None,
@@ -638,7 +643,7 @@ class ActorCriticPolicy(MuiltiNetwork):
                                     log_gradients=False, log_weights=False,
                                     accumulate_grads=accumulate_grads)
 
-    def training_model(self, current_episode=0, current_step=0, num_episodes=100, train_timing=None, done=False, batch_size=1, repeat_train=1):
+    def training_model(self, current_episode=0, current_step=0, num_episodes=100, train_timing=None, done=False, batch_size=1, repeat_train=1,**kwargs):
 
         is_collect_data = False
         for i in range(repeat_train):
@@ -663,7 +668,7 @@ class ActorCriticPolicy(MuiltiNetwork):
             self.save_or_sync_weights()
 
     def play(self, num_episodes, batch_size=1, min_replay_samples=1, print_progess_frequency=5, training=True, train_timing='on_episode_start', train_every_nstep=1, repeat_train=1,
-             need_render=True):
+             need_render=True,**kwargs):
         if train_timing not in ['on_episode_start', 'on_episode_end', 'on_step_end']:
             raise ValueError('Only on_episode_start,on_step_end are valid  train_timing options')
 
@@ -712,10 +717,9 @@ class ActorCriticPolicy(MuiltiNetwork):
                 # # Train on_step_start
                 # if training and train_timing == 'on_step_start' and t % train_every_nstep == 0:
                 #     self.training_model(i_episode, t,num_episodes=num_episodes, repeat_train=repeat_train, batch_size=batch_size)
-                if self.current_screen.shape != self.prev_screen.shape:
-                    self.prev_screen = self.current_screen.copy()
 
-                action = self.select_action((2 * self.current_screen + self.prev_screen) / 3.0, model_only=True)
+
+                action = self.select_action(self.get_observation(), model_only=True)
                 observation, reward, done, info = self.get_rewards(action)
 
                 self.total_rewards += reward
@@ -774,7 +778,7 @@ class ActorCriticPolicy(MuiltiNetwork):
         self.env.close()
 
     def learn(self, num_episodes, batch_size=1, min_replay_samples=1, print_progess_frequency=5, train_timing='on_episode_start', train_every_nstep=1, repeat_train=1,
-              need_render=True):
+              need_render=True,**kwargs):
         self.play(num_episodes=num_episodes, batch_size=batch_size, min_replay_samples=min_replay_samples, print_progess_frequency=print_progess_frequency, training=True,
                   train_timing=train_timing, train_every_nstep=train_every_nstep,
                   repeat_train=repeat_train, need_render=need_render)
@@ -1179,9 +1183,9 @@ class PPOPolicy(ActorCriticPolicy):
     """Proximal Policy Optimization
     """
 
-    def __init__(self, actor: Layer, critic: Layer, env: gym.Env, use_experience_replay=False, memory_length: int = 10000
+    def __init__(self, actor: Layer, critic: Layer, env: gym.Env,state_shape=None ,use_experience_replay=False, memory_length: int = 10000
                  , gamma=0.9, beta=0.01, use_gae=False, gae_lambda=0.95, normalize_advantages=False, clip_grad_norm=0.5, name='ppo') -> None:
-        super(PPOPolicy, self).__init__(actor=actor, critic=critic, env=env, action_strategy=ActionStrategy.OnPolicy, gamma=gamma, use_experience_replay=use_experience_replay,
+        super(PPOPolicy, self).__init__(actor=actor, critic=critic, env=env,state_shape=state_shape, action_strategy=ActionStrategy.OnPolicy, gamma=gamma, use_experience_replay=use_experience_replay,
                                         replay_unit='episode',
                                         memory_length=memory_length, name=name)
         self.state = self.env.reset()
@@ -1261,8 +1265,13 @@ class PPOPolicy(ActorCriticPolicy):
             Ratio1 = []
             Ratio2 = []
             LogProbs = []
-            # NewBest = []
-            # OldBest = []
+
+            #maske lstm stateful
+            for module in self.actor.model.modules():
+                class_name = module.__class__.__name__.lower()
+                if 'lstm' in class_name or 'gru' in class_name:
+                    if hasattr(module, 'stateful'):
+                        module.stateful = False
 
             for i in reversed(range(len(reward_pool))):
                 running_add = running_add * self.gamma + reward_pool[i]
@@ -1270,22 +1279,6 @@ class PPOPolicy(ActorCriticPolicy):
                 value = self.critic.model(state_pool[i])[:, action_pool[i]]
                 V.insert(0, value)
 
-                new_probs = self.actor(state_pool[i])
-                new_m = Categorical(probs=new_probs)
-                new_log_prob = new_m.log_prob(to_tensor(action_pool[i]))
-                LogProbs.insert(0, new_log_prob)
-
-                # new_best_action=argmax(new_probs,1).item()
-                # old_best_action=argmax(probs_pool[i],1).item()
-                # OldBest.insert(0, old_best_action)
-                # NewBest.insert(0, new_best_action)
-                # current_action=action_pool[i]
-
-                ratio1 = exp(new_log_prob - log_prob_pool[i])
-                ratio2 = clip(ratio1, 1.0 - 0.2, 1.0 + 0.2)
-                Ratio1.insert(0, ratio1)
-                Ratio2.insert(0, ratio2)
-
             R = to_tensor(R).detach()
 
             V_ = V.copy()[1:]
@@ -1303,11 +1296,25 @@ class PPOPolicy(ActorCriticPolicy):
                 clipped_gaes = self.calculate_gae(clipped_value, V_, R, done_pool, self.gamma, self.gae_lambda)
                 advantages = max(gaes, clipped_gaes)
                 if self.normalize_advantages:
+                    #only divide std
                     advantages = advantages / (advantages.std() + 1e-5)
             else:
                 advantages = max(V - R, clipped_value - R)
 
             critic_loss = (advantages ** 2).mean()
+
+            #need forward calculate for lstm-based actor
+            for i in range(len(reward_pool)):
+                new_probs = self.actor(state_pool[i])
+                new_m = Categorical(probs=new_probs)
+                new_log_prob = new_m.log_prob(to_tensor(action_pool[i]))
+                LogProbs.insert(0, new_log_prob)
+
+                ratio1 = exp(new_log_prob - log_prob_pool[i])
+                ratio2 = clip(ratio1, 1.0 - 0.2, 1.0 + 0.2)
+                Ratio1.insert(0, ratio1)
+                Ratio2.insert(0, ratio2)
+
             LogProbs = concate(LogProbs, axis=0)
 
             Ratio1 = stack(Ratio1, axis=0)
@@ -1318,21 +1325,19 @@ class PPOPolicy(ActorCriticPolicy):
             surr1 = Ratio1 * (advantages.detach())
             surr2 = Ratio2 * (advantages.detach())
             actor_loss = None
-            if last_done:
-                actor_loss = (-1 * LogProbs * advantages.detach()).mean()
-            else:
-                actor_loss = (-1 * min(surr1, surr2)).mean()
+            actor_loss = (-1 * minimum(surr1,surr2)).mean()
 
             return critic_loss, actor_loss, advantages, R, V
         except Exception as e:
             print(e)
 
     def save_or_sync_weights(self):
-        if (self.train_steps + 1) % 5 == 0:
+        if self.current_epoch <=1 or  (self.train_steps + 1) % 5 == 0:
             self.save_model(save_path=self.training_context['save_path'])
             self.critic.model.to(get_device())
             self.actor.model.to(get_device())
             self.old_actor.load_state_dict(self.actor.model.state_dict())
+            self.old_actor.eval()
             self.actor.model.train()
 
     def training_model(self, current_episode=0, current_step=0, num_episodes=100, train_timing='on_step_end', done=False, repeat_train=1):
@@ -1354,17 +1359,24 @@ class PPOPolicy(ActorCriticPolicy):
                 actor_loss = to_tensor(0.0)
                 critic_loss = to_tensor(0.0)
 
-                if (len(reward_pool) <= 3 or self.done == True or random.random() < 0.1) and len(self.memory) > 5 and self.current_epoch > 5:
-                    for i in range(8 if done else 2):
+                if self.done == True and len(self.memory) > 5 and self.current_epoch > 0:
+                    for module in self.actor.model.modules():
+                        class_name=module.__class__.__name__.lower()
+                        if 'lstm' in class_name or 'gru' in class_name:
+                            if hasattr(module,'stateful'):
+                                module.stateful=False
+
+                    for i in range(8):
                         try:
                             self.train_steps += 1
                             self.current_batch += 1
                             self.steps = self.train_steps
-                            self.done = False
+
+
 
                             # self.do_on_batch_start()
 
-                            batch_data = self.memory.get_samples(builtins.min(64 if done else 32, len(self.memory)))
+                            batch_data = self.memory.get_samples(builtins.min(64, len(self.memory)))
                             batch_return = batch_data['return']
                             batch_state = batch_data['state']
                             batch_action = batch_data['action']
@@ -1374,7 +1386,7 @@ class PPOPolicy(ActorCriticPolicy):
                             batch_return = to_tensor(batch_return)
                             batch_advantages = (batch_return.detach() - batch_value)
 
-                            batch_critic_loss = (batch_advantages ** 2).mean()
+                            batch_critic_loss =(batch_advantages ** 2).mean()
                             self.critic.current_loss = batch_critic_loss
                             self.critic.batch_loss_history.collect('critic_loss', self.train_steps, batch_critic_loss.copy().detach())
                             self.critic.batch_metric_history.collect('critic_rmse', self.train_steps, (batch_critic_loss.copy().detach()).sqrt())
@@ -1382,19 +1394,19 @@ class PPOPolicy(ActorCriticPolicy):
                             self.critic.current_loss = 0.0
                             batch_actor_loss = to_tensor(0.0)
                             for j in range(len(batch_state)):
-                                old_batch_probs = self.old_actor(to_tensor(batch_state[j]))
-                                old_m = Categorical(probs=old_batch_probs)
-                                old_log_prob = old_m.log_prob(to_tensor(batch_action[j]).long())
+                                # old_batch_probs = self.old_actor(to_tensor(batch_state[j]))
+                                # old_m = Categorical(probs=old_batch_probs)
+                                old_batch_probs = batch_log_prob[j].long().detach()
                                 new_batch_probs = self.actor(to_tensor(batch_state[j]))
                                 new_m = Categorical(probs=new_batch_probs)
                                 new_log_prob = new_m.log_prob(to_tensor(batch_action[j]).long())
 
                                 # entropy_loss = entropy_loss + new_m.entropy().mean()
-                                ratio = exp(old_batch_probs - old_batch_probs)
+                                ratio = exp(new_log_prob - old_batch_probs)
                                 cliprange = 0.2
                                 surr1 = ratio * (batch_advantages[j].detach())
                                 surr2 = clip(ratio, 1.0 - cliprange, 1.0 + cliprange) * (batch_advantages[j].detach())
-                                batch_actor_loss = batch_actor_loss - min(surr1, surr2).mean()
+                                batch_actor_loss = batch_actor_loss - (minimum(surr1,surr2)).mean()
                             self.actor.current_loss = batch_actor_loss / len(batch_state)
                             self.actor.batch_loss_history.collect('actor_loss', self.train_steps, batch_actor_loss.copy() / len(batch_state))
                             # self.actor.batch_loss_history.collect('entropy_loss', self.train_steps, (-0.01 * entropy_loss).copy().detach() / (len(state_pool) - 1))
@@ -1404,6 +1416,7 @@ class PPOPolicy(ActorCriticPolicy):
 
                         except Exception as e:
                             print(e)
+                            PrintException()
 
                     # self.critic.print_batch_progress(8 if done else 2)
                     # self.actor.print_batch_progress(8 if done else 2)
@@ -1417,48 +1430,6 @@ class PPOPolicy(ActorCriticPolicy):
                         self.actor.steps = self.train_steps
                         self.critic.steps = self.train_steps
                         self.done = done
-                        # self.do_on_batch_start()
-
-                        # gae = 0
-
-                        # last_done = done_pool[-1]
-                        # next_value = 0.0 if last_done else self.critic(state_pool[-1])[:, action_pool[-1]]
-                        # for t in list(range(len(state_pool)))[::-1]:
-                        #     td_error1 = 0
-                        #     td_error2 = 0
-                        #
-                        #     if done_pool[t] == True or reward_pool[t] > 0.15 or reward_pool[t] < -0.15:
-                        #         keep.extend([t, t - 1, t - 2, t - 3, t - 4])
-                        #     elif random.random() < 0.05:
-                        #         keep.extend([t, t - 1])
-                        #
-                        #     value = self.critic(state_pool[t])[:, action_pool[t]]
-                        #     # current_return = reward_pool[t] + self.gamma * (1 - done_pool[t]) * next_reward
-                        #
-                        #     # td_error1 = reward_pool[t] + self.gamma * (1 - done_pool[t]) * next_value - value
-                        #     # clip_range = [current_return.item() * (1 - 0.2), current_return.item() * (1 + 0.2)]
-                        #     clipped_value = next_value + clip(value - next_value, -0.2, 0.2)
-                        #
-                        #     delta = reward_pool[t] + self.gamma * (1 - done_pool[t]) * next_value - value
-                        #
-                        #     gae = delta + self.gamma * self.gae_lambda * (1 - done_pool[t]) * gae
-                        #
-                        #     # td_error2 = current_return + self.gamma * (1 - done_pool[t]) * next_value -clip(value, builtins.min(clip_range), builtins.max(clip_range))
-                        #     # gae = gae * gae_lambda * self.gamma * (1 - done_pool[t]) +td_error1# max(td_error1, td_error2)
-                        #     R.insert(0, gae + value)
-                        #     V.insert(0, value)
-                        #     V2.insert(0, clipped_value)
-                        #
-                        #     next_value = value
-                        #
-                        # R = concate(R, axis=0)
-                        # # advantages=(advantages-advantages.mean())/advantages.std()
-                        # V = concate(V, axis=0)
-                        # V2 = concate(V2, axis=0)
-                        #
-                        # advantages = (R.detach() - where(V > V2, V, V2))
-                        # # advantages=(advantages-advantages.mean())/(advantages.std()+1e-7)
-                        # critic_loss = (advantages ** 2).mean()
 
                         critic_loss, actor_loss, advantages, R, V = self.calculate_discounted_returns()
                         self.critic.current_loss = critic_loss
@@ -1481,33 +1452,10 @@ class PPOPolicy(ActorCriticPolicy):
                         #
                         # keep = list(filter(not_negative, list(sorted((set(keep))))))
                         # for idx in keep:
-                        for idx in list(range(len(state_pool)))[::-1]:
-                            self.memory.collect('return', R[idx])
-                            self.memory.collect('state', to_numpy(state_pool[idx]))
-                            self.memory.collect('action', action_pool[idx])
-                            self.memory.collect('log_prob', log_prob_pool[idx])
 
-                        if len(self.memory.get('return')) > 3000:
-                            for k in self.memory.keys():
-                                self.memory[k] = self.memory[k][1000:]
-                        # print('tota; length:',len(self.memory.value_list[0]),keep)
 
-                        # print(list(self.critic.model[-1].named_parameters()))
-                        # self.critic.print_batch_progress(1)
-                        # self.actor.model.to(get_device())
-                        # for j in range(len(state_pool)):
-                        #     state = state_pool[j].to(get_device())
-                        #     new_probs = self.actor(state)
-                        #     new_m = Categorical(probs=new_probs)
-                        #     new_log_prob = new_m.log_prob(to_tensor(action_pool[j]))
-                        #     # entropy_loss = entropy_loss + new_m.entropy().mean()
-                        #     ratio = exp(new_log_prob - log_prob_pool[j])
-                        #     cliprange = 0.2
-                        #     surr1 = ratio * (advantages[j].detach())
-                        #     surr2 = clip(ratio, 1.0 - cliprange, 1.0 + cliprange) * (advantages[j].detach())
-                        #     actor_loss = actor_loss - min(surr1, surr2)
-                        #
-                        #
+
+
 
                         self.actor.current_loss = actor_loss
                         self.actor.batch_loss_history.collect('actor_loss', self.train_steps, actor_loss.copy().detach())
@@ -1516,8 +1464,17 @@ class PPOPolicy(ActorCriticPolicy):
                         self.update_actor(self.current_epoch, self.train_steps, self.total_epoch, 1, 'on_step_end', self.done, accumulate_grads=False)
                         self.actor.current_loss = 0.0
                         # self.actor.print_batch_progress(1)
+                        if  self.done==True:
+                            self.save_or_sync_weights()
+                            #self.memory.reset()
+
+                        for idx in list(range(len(state_pool)))[::-1]:
+                            if self.rollout.reward[idx]!=0:
+                                self.memory.collect('return', advantages[idx].item()+V[idx].item())
+                                self.memory.collect('state', to_numpy(state_pool[idx]))
+                                self.memory.collect('action', action_pool[idx])
+                                self.memory.collect('log_prob', log_prob_pool[idx])
 
-                        self.save_or_sync_weights()
 
                         self.rollout.reset()
                         del log_prob_pool
@@ -1531,6 +1488,8 @@ class PPOPolicy(ActorCriticPolicy):
 
                     except Exception as e:
                         print(e)
+                        PrintException()
 
             except Exception as e:
                 print(e)
+                PrintException()
